unit AdvPrinter;

{TAdvPrinter (C) by CEVis Systems. Version 0.9

 Totally Free. Send me a e-Mail if it works or
 if you improve something. If not, send none.

 Contact: lnie@bilfinger.de

}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Printers, ExtCtrls, StdCtrls, ComCtrls;

type TAdvPrinterOrientation = (apoPortrait,apoLandscape);

type
  TAdvPrinter = class(TComponent)
    Constructor Create(AOwner : TComponent); override;
    Destructor Destroy; override;
  private
    { Private-Deklarationen }
    FSheet,
    FSaveSheet   : TBitmap;
    FPreviews	 : TList;
    FViewControl : TPaintBox;
    FNextControl, FPrevControl : TButton;
    FZoomControl : TTrackbar;
    FOutControl  : TLabel;
    FViewSize    : integer;
    FPreviewPage : integer;
    FPreviewonly : boolean;
    FPageCount   : integer;

    function FPrintDevice : TPrinter;
    procedure ChangeViewSize(Sender : TObject);
    procedure SetupBitmap(ASheet : TBitmap);
    procedure SetupBitmapParams(ASheet : TBitmap);
    function CalcX(x : integer) : integer;
    function CalcY(y : integer) : integer;
    procedure Preview(Sender : TObject);
    procedure SetFOrientation(Value : TAdvPrinterOrientation);
    function GetFOrientation : TAdvPrinterOrientation;
    procedure SetFBrush(Value : TBrush);
    function GetFBrush : TBrush;
    procedure SetFPen(Value : TPen);
    function GetFPen : TPen;
    procedure SetFFont(Value : TFont);
    function GetFFont : TFont;
    function GetString(Index : integer) : string;
    procedure SetString(Index : integer; Value : string);
    function Getinteger(Index : integer) : integer;
    procedure SetInteger(Index : integer; Value : integer);
    function Getboolean(Index : integer) : boolean;
    function PageExtends : TPoint;
    function PageRect    : TRect;
    function ShadowRect  : TRect;
  protected
    { Protected-Deklarationen }
  public
    { Public-Deklarationen }
    //additional Methods
    procedure Reset; //Resets the Object: Previews=0 and all Counters=0
    // copied Printer methods
    procedure BeginDoc;
    procedure EndDoc;
    procedure Abort;
    procedure NewPage;

    //Directly mapped Canvas methods, add some more if you like...
    procedure Rectangle(const Rect: TRect);
    procedure StretchDraw(const Rect: TRect; Graphic: TGraphic );
    procedure LineTo(X, Y: Integer);
    procedure MoveTo(X, Y: Integer);
    procedure TextOut(X, Y: Integer; const Text: string);
    function TextHeight(const Text: string): Integer;
    function TextWidth(const Text: string): Integer;

    property Printing    : boolean index 0 read GetBoolean;
    property Title	 : string index 0 read GetString write SetString;
    property Orientation : TAdvPrinterOrientation read GetFOrientation write SetFOrientation;
    property PageHeight  : integer index 0 read GetInteger;
    property PageWidth   : integer index 1 read GetInteger;
    property PageNumber  : integer index 2 read GetInteger;
    property PageCount   : integer index 4 read GetInteger;
    property Font	 : TFont read GetFFont write SetFFont;
    property Pen	 : TPen read GetFPen write SetFPen;
    property Brush	 : TBrush read GetFBrush write SetFBrush;

    // "The Printer"
    property PrintDevice : TPrinter read FPrintDevice;

    // "The Previews"

    property Previews    : TList read FPreviews;
  published
    { Published-Deklarationen }
    property ViewControl : TPaintBox read FViewControl write FViewControl;
    property NextControl : TButton read FNextControl write FNextControl;
    property PrevControl : TButton read FPrevControl write FPrevControl;
    property ZoomControl : TTrackBar read FZoomControl write FZoomControl;
    property OutControl  : TLabel read FOutControl write FOutControl;
    property ViewSize    : integer index 3 read GetInteger write SetInteger;
    property PreviewOnly : boolean index 1 read FPreviewOnly write FPreviewonly;
  end;


procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Eigene', [TAdvPrinter]);
end;

Constructor TAdvPrinter.Create;

begin
 inherited Create(AOwner);
 FPreviews:=TList.Create;
 FPreviewPage:=-1;
 FPageCount:=0;
end;

Destructor TAdvPrinter.Destroy;

var i : integer;

begin
 for i:=0 to FPreviews.Count-1 do
  TBitmap(FPreviews[i]).Free;
 FPreviews.Free;
 inherited Destroy;
end;

function TAdvPrinter.FPrintDevice;

begin
 result:=printer;
end;

procedure TAdvPrinter.Reset;

var i : integer;

begin
 for i:=0 to FPreviews.Count-1 do
  TBitmap(FPreviews[i]).Free;
 FPreviews.Clear;
 if Assigned(FViewControl) and
    Assigned(FNextControl) and
    Assigned(FPrevControl) and
    Assigned(FZoomControl) and
    Assigned(FOutControl) then
 begin
  FViewControl.OnPaint:=PreView;
  FPrevControl.OnClick:=Preview;
  FNextControl.OnClick:=Preview;
  FZoomControl.OnChange:=ChangeViewSize;
 end;
 FPreviewpage:=-1;
 FPageCount:=0;
end;

procedure TAdvPrinter.SetFBrush;

begin
 FPrintDevice.Canvas.Brush:=Value;
end;

function TAdvPrinter.GetFBrush;

begin
 result:=FPrintDevice.Canvas.Brush;
end;

procedure TAdvPrinter.SetFPen;

begin
 FPrintDevice.Canvas.Pen:=Value;
end;

function TAdvPrinter.GetFPen;

begin
 result:=FPrintDevice.Canvas.Pen;
end;

procedure TAdvPrinter.SetFFont;

begin
 FPrintDevice.Canvas.Font:=Value;
end;

function TAdvPrinter.GetFFont;

begin
 result:=FPrintDevice.Canvas.Font;
end;

procedure TAdvPrinter.StretchDraw;

var Arect : Trect;

begin
 if not FPreviewOnly then
  FPrintDevice.Canvas.StretchDraw(Rect,Graphic);
 SetupBitMapParams(FSheet);
 ARect.Left:=CalcX(Rect.Left);
 ARect.Top:=CalcY(Rect.top);
 ARect.Right:=CalcX(Rect.Right);
 ARect.Bottom:=CalcX(Rect.Bottom);
 with FSheet.Canvas do
 begin
  StretchDraw(Arect,Graphic);
 end;
end;

procedure TAdvPrinter.Rectangle;

var ARect : Trect;

begin
 if not FPreviewOnly then
  FPrintDevice.Canvas.Rectangle(Rect);
 SetupBitMapParams(FSheet);
 ARect.Left:=CalcX(Rect.Left);
 ARect.Top:=CalcY(Rect.top);
 ARect.Right:=CalcX(Rect.Right);
 ARect.Bottom:=CalcX(Rect.Bottom);
 with FSheet.Canvas do
 begin
  Rectangle(Arect);
 end;
end;

procedure TAdvPrinter.MoveTo;

begin
 if not FPreviewOnly then
  FPrintDevice.Canvas.MoveTo(x,y);
 SetupBitMapParams(FSheet);
 with FSheet.Canvas do
 begin
  MoveTo(CalcX(x),CalcY(y));
 end;
end;

procedure TAdvPrinter.LineTo;

begin
 if not FPreviewOnly then
  FPrintDevice.Canvas.LineTo(x,y);
 SetupBitMapParams(FSheet);
 with FSheet.Canvas do
 begin
  LineTo(CalcX(x),CalcY(y));
 end;
end;

procedure TAdvPrinter.TextOut;

begin
 if not FPreviewOnly then
  FPrintDevice.Canvas.TextOut(x,y,Text);
 SetupBitmapParams(FSheet);
 with FSheet.Canvas do
 begin
  TextOut(CalcX(x),CalcY(y),text);
 end;
end;

function TAdvPrinter.TextWidth;

begin
 result:=FPrintDevice.Canvas.TextWidth(Text);
end;

function TAdvPrinter.TextHeight;

begin
 result:=FPrintDevice.Canvas.TextHeight(Text);
end;

procedure TAdvPrinter.SetInteger;

begin
 case index of
 3 : FViewSize:=Value;
 end;
end;

function TAdvPrinter.GetInteger;

begin
 case index of
 0 : result:=FPrintDevice.PageHeight;
 1 : result:=FPrintDevice.PageWidth;
 2 : result:=FPrintDevice.PageNumber;
 3 : result:=FViewSize;
 4 : result:=FPageCount;
 end;
end;

procedure TAdvPrinter.SetString;

begin
 case index of
 0 : FPrintDevice.Title:=Value;
 end;
end;

function TAdvPrinter.GetString;

begin
 case index of
 0 : result:=FPrintDevice.Title;
 end;
end;

function TAdvPrinter.GetBoolean;

begin
 case index of
 0 : result:=FPrintDevice.Printing;
 end;
end;

procedure TAdvPrinter.SetFOrientation;

begin
 case value of
  apoPortrait  : FPrintDevice.Orientation:=poPortrait;
  apolandscape : FPrintDevice.Orientation:=poLandscape;
 end;
end;

function TAdvPrinter.GetFOrientation;

begin
 case FPrintDevice.Orientation of
  poPortrait  : result:=apoPortrait;
  poLandscape : result:=apoLandscape;
 end;
end;

function TAdvPrinter.CalcX;

begin
 result:=round(PageExtends.x/pagewidth*x);
end;

function TAdvPrinter.CalcY;

begin
 result:=round(PageExtends.y/pageHeight*y);
end;

function TAdvPrinter.PageExtends;

begin
 if Assigned(FViewControl) then
 begin
  case Orientation of
  apoPortrait  : begin
                  result.y:=round(FViewControl.Height*0.95*FViewSize/100);
                  result.x:=round(result.y*PageWidth/PageHeight);
                 end;
  apoLandscape : begin
                  result.x:=round(FViewControl.Width*0.95*FViewSize/100);
                  result.y:=round(result.x*PageHeight/PageWidth);
                 end;
  end;
 end else
 begin
  result.x:=32;
  result.y:=32;
 end;
end;

function TAdvPrinter.ShadowRect;

var ARect : TRect;
    Dx,Dy : integer;
const shd1 = 0.01;

begin
 ARect:=PageRect;
 dx:=round((ARect.Right-ARect.Left)*shd1);
 dy:=round((ARect.Bottom-ARect.Top)*shd1);
 if dx>dy then dy:=dx else dx:=dy;
 result:=Rect(ARect.Left+dx,
 	      ARect.Top+dy,
              ARect.Right+dx,
              ARect.Bottom+dy);
end;

function TAdvPrinter.PageRect;

var extd : TPoint;

begin
 if Assigned(FViewControl) then

 with FViewControl do
 begin
  Extd:=PageExtends;
  result:=Rect((width-extd.x) div 2,
               (Height-extd.y) div 2,
                width-(width-extd.x) div 2,
                Height-(Height-extd.y) div 2);
 end else
 begin
  result:=Rect(10,10,20,20);
 end;
end;

procedure TAdvPrinter.SetupBitmap;

var Extd : TPoint;

begin
 Extd:=PageExtends;
 ASheet.width:=Extd.x;
 ASheet.Height:=Extd.y;
 ASheet.Canvas.Pen.Color:=clBlack;
 ASheet.Canvas.Brush.Style:=bsClear;
 ASheet.Transparent:=False;
 ASheet.Dormant;
end;

procedure TAdvPrinter.SetupBitmapParams;

var s,h : integer;

function NoTT(AFont : TFont) : boolean;

begin
 result:=(Font.name='MS Sans Serif');
end;

begin
 With ASheet.Canvas do
 begin
  Font.Name:=FPrintDevice.Canvas.Font.Name;
  if NoTT(Font) then // no TrueType...
   Font.Name:='Arial Narrow';
  Font.Size:=FPrintDevice.Canvas.Font.Size;
  Font.Pitch:=FPrintDevice.Canvas.Font.Pitch;
  Brush.Style:=FPrintDevice.Canvas.Brush.Style;
  Brush.Color:=FPrintDevice.Canvas.Brush.Color;
  Pen.Color:=FPrintDevice.Canvas.Pen.Color;
  Pen.Style:=FPrintDevice.Canvas.Pen.Style;
  h:=round(ASheet.Height/PageHeight*FPrintDevice.Canvas.Font.Height);
  Font.Height:=h;
 end;
end;

procedure TAdvPrinter.BeginDoc;

begin
 Reset;
 if not FPreviewOnly then
  FPrintDevice.BeginDoc
 else
 begin
  FPrintDevice.BeginDoc;  //Re-init printer... not tidy, I know
  FPrintDevice.Abort;
 end;
 FSheet:=TBitmap.Create;
 SetupBitmap(FSheet);
 FPreViews.add(FSheet);
 Inc(FPreviewPage);
 FPageCount:=FPreviews.Count;
end;

procedure TAdvPrinter.EndDoc;

begin
 if not FPreviewOnly then
  FPrintDevice.EndDoc;
end;

procedure TAdvPrinter.NewPage;

begin
 if not FPreviewOnly then
  FPrintDevice.NewPage;
 FSheet:=TBitmap.Create;
 SetupBitmap(FSheet);
 FPreviews.add(FSheet);
 Inc(FPreviewPage);
 FPageCount:=FPreviews.Count;
end;

procedure TAdvPrinter.Abort;

begin
 if not FPreviewOnly then
  FPrintDevice.Abort;
end;

procedure TAdvPrinter.PreView;

var ARect : Trect;

begin
 if (Sender=FViewControl) then FPreviewPage:=0; //Start with first...

 if (Sender=FNextControl) and
    (FPreviewPage<FPreviews.Count-1) then inc(FPreviewPage); // Next Page

 if (Sender=FPrevControl) and // Previous Page
    (FPreviewPage>0) then dec(FPreviewPage);

 FZoomControl.Min:=0;
 FZoomControl.Max:=100; // it overwrites the ones set by IDE!

 FOutControl.Caption:='Seiten: '+IntToStr(FPreViewPage+1)+'/'+
 		      IntToStr(FPageCount)+' bei '+
 		      inttoStr(FViewSize)+'%';

 if FPreviews.Count>0 then
  With FViewControl.Canvas do
  begin
   Brush.Style:=bsSolid;
   Brush.Color:=FViewControl.Color;
   ARect:=Bounds(0,0,FViewControl.width,FViewControl.Height);
   FillRect(ARect);
   Brush.Color:=clGray;
   FillRect(ShadowRect);
   if (sender<>FZoomControl) then
   begin
    ARect:=Bounds(0,0,PageExtends.x,PageExtends.y);
    CopyRect(PageRect,TBitmap(FPreviews[FPreviewPage]).Canvas,ARect);
   end else
   begin
    StretchDraw(PAgeRect,FSaveSheet);
    FSaveSheet.Free;
   end;
  end;
end;

procedure TAdvPrinter.ChangeViewSize;

begin
 if (Sender=FzoomControl) then
   FViewSize:=FZoomControl.Position;
 if FPreviews.Count>0 then
 begin
  FSaveSheet:=TBitMap.Create;
  SetupBitmap(FSaveSheet);
  FSaveSheet.Assign(TBitmap(FPreviews[FPreviewPage]));
  PreView(sender);
 end;
end;

end.
